@@ -26,6 +26,7 @@
 #include <time.h>
 
 #include "btobex.h"
+#include "btctl-types.h"
 #include "btctl-marshal.h"
 #include "obexsdp.h"
 
@@ -90,13 +91,12 @@ btctl_obex_class_init (BtctlObexClass *klass)
 			G_SIGNAL_RUN_FIRST,
 			G_STRUCT_OFFSET(BtctlObexClass, put),
 			NULL /* accu */, NULL,
-			btctl_marshal_VOID__STRING_STRING_POINTER_UINT_UINT,
+			btctl_marshal_VOID__STRING_STRING_POINTER_UINT,
 			G_TYPE_NONE,
-			5,
+			4,
 			G_TYPE_STRING,
 			G_TYPE_STRING,
-			G_TYPE_POINTER,
-			G_TYPE_UINT,
+			BTCTL_TYPE_OBEX_DATA,
 			G_TYPE_UINT);
 
 	btctl_obex_signals[PROGRESS_SIGNAL] =
@@ -235,11 +235,11 @@ btctl_obex_emit_progress (BtctlObex *bo, gchar *bdaddr)
 
 static void
 btctl_obex_emit_put (BtctlObex *bo, gchar *bdaddr, guchar *fname,
-				gpointer data, guint len, guint timestamp)
+				BtctlObexData* data, guint timestamp)
 {
 	g_signal_emit (G_OBJECT (bo),
 		btctl_obex_signals[PUT_SIGNAL],
-		0, bdaddr, fname, data, len, timestamp);
+		0, bdaddr, fname, data, timestamp);
 }
 
 static void
@@ -320,7 +320,11 @@ put_done (BtctlObex *bo, BtctlObexserverSource *src, BtctlObexserverEvent *evt)
 		name = g_strdup ("Unknown object");
 	}
 	if (body) {
-		btctl_obex_emit_put (bo, src->peer_name, name, (gpointer) body, body_len, timestamp);
+		BtctlObexData *obexData = g_new0(BtctlObexData, 1);
+		obexData->body = body;
+		obexData->body_len = body_len;
+
+		btctl_obex_emit_put (bo, src->peer_name, name, obexData, timestamp);
 	} else {
 		g_warning ("btctl_obex::put_done: no body received, ignoring");
 	}
